require "./util/worker_pool"
require "http/client"
require "json"

worker_pool = WorkerPool(BulkImport, Feature).new do |bulk, feature|
  bulk.enqueue feature
end

worker_pool.on_close do |bulk|
  bulk.flush
end

worker_pool.start do
  client = HTTP::Client.new("localhost", 9200)
  BulkImport.new(client)
end

HTTP::Client.delete "http://localhost:9200/citylots"

class BulkImport
  def initialize(@http : HTTP::Client, @batch_size = 1000)
    @batch = Array(Feature).new(@batch_size)
  end

  def enqueue(feature)
    @batch << feature
    flush if @batch.size >= @batch_size
  end

  INDEX_HEADER = {"index" => ({"_index" => "citylots", "_type" => "feature"})}.to_json

  def flush
    return if @batch.empty?

    body = String.build do |str|
      @batch.each do |feature|
        str.puts INDEX_HEADER
        str.puts feature.to_json
      end
    end
    @http.post "/_bulk", body: body
    @batch.clear
  end
end

File.open("citylots.json") do |f|
  parser = JSON::PullParser.new(f)
  count = 0
  parser.on_key("features") do
    parser.read_array do
      parser.on_key("properties") do
        count += 1
        if (count % 1000) == 0
          puts count
        end
        feature = Feature.new(parser)
        worker_pool.enqueue feature
      end
    end
  end
end

worker_pool.close

class Feature
  JSON.mapping(
    mapblklot: {type: String, key: "MAPBLKLOT"},
    blklot: {type: String, key: "BLKLOT"},
    block_num: {type: String, key: "BLOCK_NUM"},
    lot_num: {type: String, key: "LOT_NUM"},
    from_st: {type: String?, key: "FROM_ST"},
    to_st: {type: String?, key: "TO_ST"},
    street: {type: String?, key: "STREET"},
    st_type: {type: String?, key: "ST_TYPE"},
    odd_even: {type: String?, key: "ODD_EVEN"}
  )
end
