require "mysql"
require "json"

WORKERS = 8

INSERT_SQL = <<-SQL
  INSERT INTO citylots(mapblklot, blklot, block_num, lot_num, from_st, to_st, street, st_type, odd_even)
  VALUES (?, ?, ?, ?, ?, ?, ?, ?, ?)
  SQL

puts "Opening database"
DB.open "mysql://root@localhost/codecamp" do |db|
  puts "Recreating table"
  db.exec "DROP TABLE IF EXISTS citylots"
  db.exec <<-SQL
  CREATE TABLE citylots (
    mapblklot VARCHAR(64),
    blklot VARCHAR(64),
    block_num VARCHAR(64),
    lot_num VARCHAR(64),
    from_st VARCHAR(64),
    to_st VARCHAR(64),
    street VARCHAR(256),
    st_type VARCHAR(256),
    odd_even CHAR
  )
  SQL

  features_channel = Channel(Feature).new(100)
  end_channel = Channel(Nil).new

  WORKERS.times do
    spawn do
      begin
        loop do
          feature = features_channel.receive
          db.exec INSERT_SQL,
            feature.mapblklot, feature.blklot,
            feature.block_num, feature.lot_num,
            feature.from_st, feature.to_st,
            feature.street, feature.st_type, feature.odd_even
        end
      rescue Channel::ClosedError
        end_channel.send nil
      end
    end
  end

  puts "Opening JSON file"
  File.open("citylots.json") do |f|
    parser = JSON::PullParser.new(f)
    puts "Parsing JSON file"
    parser.read_object do |key|
      if key == "features"
        count = 0
        parser.read_array do
          count += 1
          if (count % 1000) == 0
            puts count
          end
          parser.read_object do |key|
            if key == "properties"
              feature = Feature.new(parser)
              features_channel.send feature
            else
              parser.skip
            end
          end
        end
      else
        parser.skip
      end
    end
  end

  features_channel.close
  WORKERS.times { end_channel.receive }
end

class Feature
  JSON.mapping(
    mapblklot: {type: String, key: "MAPBLKLOT"},
    blklot: {type: String, key: "BLKLOT"},
    block_num: {type: String, key: "BLOCK_NUM"},
    lot_num: {type: String, key: "LOT_NUM"},
    from_st: {type: String?, key: "FROM_ST"},
    to_st: {type: String?, key: "TO_ST"},
    street: {type: String?, key: "STREET"},
    st_type: {type: String?, key: "ST_TYPE"},
    odd_even: {type: String?, key: "ODD_EVEN"}
  )
end
