class WorkerPool(State, Job)
  def initialize(@pool_size = 8, queue_size = 100, &block : (State, Job) ->)
    @job_channel = Channel(Job).new(queue_size)
    @end_channel = Channel(Nil).new
    @block = block
  end

  def on_close(&block : State ->)
    @on_close = block
  end

  def start(&init : -> State)
    @pool_size.times do
      spawn do
        state = init.call
        begin
          loop do
            job = @job_channel.receive
            @block.call(state, job)
          end
        rescue Channel::ClosedError
          @on_close.try &.call(state)
          @end_channel.send nil
        end
      end
    end
  end

  def enqueue(job : Job)
    @job_channel.send job
  end

  def close
    @job_channel.close
    @pool_size.times { @end_channel.receive }
  end
end
